@@ -247,40 +247,65 @@ _trafoxml_schema_defaults = {'bools': set(),
      'ints': {('Pairs', 'count')},
      'lists': {'Pair', 'Param'}}
 
-_featurexml_schema_defaults = {'bools': {
-    ('PeptideIdentification', 'higher_score_better'),
-    ('ProteinIdentification', 'higher_score_better'),
-    ('UnassignedPeptideIdentification', 'higher_score_better')},
- 'charlists': set(),
- 'floatlists': set(),
- 'floats': {('PeptideHit', 'score'),
-            ('PeptideIdentification', 'MZ'),
-            ('PeptideIdentification', 'RT'),
-            ('PeptideIdentification', 'significance_threshold'),
-            ('ProteinHit', 'score'),
-            ('ProteinIdentification', 'significance_threshold'),
-            ('SearchParameters', 'peak_mass_tolerance'),
-            ('SearchParameters', 'precursor_peak_tolerance'),
-            ('UnassignedPeptideIdentification', 'MZ'),
-            ('UnassignedPeptideIdentification', 'RT'),
-            ('UnassignedPeptideIdentification', 'significance_threshold'),
-            ('pt', 'x'), ('pt', 'y'), ('position', 'position'),
-            ('feature', 'overallquality'), ('feature', 'intensity')
-            },
- 'intlists': set(),
+_featurexml_schema_defaults = {
  'ints': {('PeptideHit', 'charge'),
-          ('PeptideIdentification', 'spectrum_reference'),
-          ('SearchParameters', 'missed_cleavages'),
-          ('UnassignedPeptideIdentification', 'spectrum_reference'),
-          ('featureList', 'count'), ('convexhull', 'nr'),
-          ('position', 'dim'), ('feature', 'spectrum_index'),
-          ('feature', 'charge'), ('quality', 'dim'), ('quality', 'quality')},
- 'lists': {'FixedModification', 'IdentificationRun',
-           'PeptideHit', 'PeptideIdentification', 'ProteinHit',
-           'UnassignedPeptideIdentification', 'VariableModification',
-           'convexhull', 'dataProcessing', 'feature', 'hposition',
-           'hullpoint', 'param', 'position', 'processingAction',
-           'pt', 'quality', 'userParam'}}
+    # ('PeptideIdentification', 'spectrum_reference'),
+    ('SearchParameters', 'missed_cleavages'),
+    # ('UnassignedPeptideIdentification', 'spectrum_reference'),
+    ('featureList', 'count'),
+    ('quality', 'dim'),
+    ('position', 'dim'),
+    ('feature', 'charge'),
+    ('convexhull', 'nr'),
+  },
+ 'floats': {('PeptideHit', 'score'),
+    ('PeptideIdentification', 'MZ'),
+    ('PeptideIdentification', 'RT'),
+    ('PeptideIdentification', 'significance_threshold'),
+    ('ProteinHit', 'coverage'),
+    ('ProteinHit', 'score'),
+    ('ProteinIdentification', 'significance_threshold'),
+    ('SearchParameters', 'peak_mass_tolerance'),
+    ('SearchParameters', 'precursor_peak_tolerance'),
+    ('UnassignedPeptideIdentification', 'MZ'),
+    ('UnassignedPeptideIdentification', 'RT'),
+    ('UnassignedPeptideIdentification', 'significance_threshold'),
+    ('featureMap', 'version'),
+    ('pt', 'x'),
+    ('pt', 'y'),
+    ('quality', 'quality'),
+    ('position', 'position'),
+    ('feature', 'overallquality'),
+    ('feature', 'intensity'),
+    },
+ 'bools': {('PeptideIdentification', 'higher_score_better'),
+    ('ProteinIdentification', 'higher_score_better'),
+    ('SearchParameters', 'peak_mass_tolerance_ppm'),
+    ('SearchParameters', 'precursor_peak_tolerance_ppm'),
+    ('UnassignedPeptideIdentification', 'higher_score_better')},
+ 'intlists': set(),
+ 'floatlists': set(),
+ 'charlists': set(),
+ 'lists': {'FixedModification',
+    'IdentificationRun',
+    'PeptideHit',
+    'PeptideIdentification',
+    'ProteinHit',
+    'ProteinIdentification',
+    'SearchParameters',
+    'UnassignedPeptideIdentification',
+    'UserParam',
+    'VariableModification',
+    'convexhull',
+    'dataProcessing',
+    'feature',
+    'hposition',
+    'hullpoint',
+    'param',
+    'position',
+    'processingAction',
+    'pt',
+    'quality'}}
 
 _tandem_schema_defaults = {'ints': {
         ('group', 'z'), ('aa', 'at')} | {('domain', k) for k in [
