@@ -8,7 +8,7 @@ Summary
 This module is used to calculate the
 electrochemical properties of polypeptide molecules.
 
-The theory behind this module is based on the Henderson-Hasselbalch
+The theory behind most of this module is based on the Henderson-Hasselbalch
 equation and was thoroughly described in a number of sources [#Aronson]_,
 [#Moore]_.
 
@@ -22,13 +22,20 @@ where the sum is taken over all ionizable groups of the polypeptide, and
 :math:`Q_i` is -1 and +1 for acidic and basic functional groups,
 respectively.
 
-Main functions
---------------
+Charge and pI functions
+-----------------------
 
   :py:func:`charge` - calculate the charge of a polypeptide
 
   :py:func:`pI` - calculate the isoelectric point of a polypeptide
 
+
+GRand AVerage of hYdropathicity (GRAVY)
+---------------------------------------
+
+  :py:func:`gravy` - calculate the GRAVY index of a polypeptide
+
+
 Data
 ----
 
@@ -47,6 +54,9 @@ Data
 
   :py:data:`pK_cterm_bjellqvist` - a set of C-terminal pK from [#Bjellqvist]_.
 
+  :py:data:`hydropathicity_KD` - a set of hydropathicity indexes from [#Kyte]_.
+
+
 References
 ----------
 
@@ -78,6 +88,11 @@ References
     correlate with polypeptide compositions. Electrophoresis 1994, 15, 529-539.
     `Link. <http://dx.doi.org/10.1002/elps.1150150171>`_
 
+.. [#Kyte] Kyte, J.; Doolittle, R. F..
+   A simple method for displaying the hydropathic character of a protein.
+   Journal of molecular biology 1982, 157 (1), 105-32.
+   `Link. <https://doi.org/10.1016/0022-2836(82)90515-0>`_
+
 -------------------------------------------------------------------------------
 
 """
@@ -96,10 +111,12 @@ References
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
 
+from __future__ import division
 from . import parser
 from .auxiliary import PyteomicsError
 from collections import Iterable, Counter
 
+
 def charge(sequence, pH, **kwargs):
     """Calculate the charge of a polypeptide in given pH or list of pHs using
     a given list of amino acid electrochemical properties.
@@ -151,6 +168,7 @@ def charge(sequence, pH, **kwargs):
     charge_list = _charge_for_dict(peptide_dict, pH_list, pK)
     return charge_list[0] if not isinstance(pH, Iterable) else charge_list
 
+
 def _prepare_charge_dict(sequence, **kwargs):
     nterm = cterm = n_aa = c_aa = None
     pK = kwargs.get('pK', pK_lehninger).copy()
@@ -226,6 +244,7 @@ def _prepare_charge_dict(sequence, **kwargs):
 
     return peptide_dict, pK
 
+
 def _charge_for_dict(peptide_dict, pH_list, pK):
     # Calculate the charge for each value of pH.
     charge_list = []
@@ -239,6 +258,7 @@ def _charge_for_dict(peptide_dict, pH_list, pK):
 
     return charge_list
 
+
 def pI(sequence, pI_range=(0.0, 14.0), precision_pI=0.01, **kwargs):
     """Calculate the isoelectric point of a polypeptide using a given set
     of amino acids' electrochemical properties.
@@ -284,7 +304,7 @@ def pI(sequence, pI_range=(0.0, 14.0), precision_pI=0.01, **kwargs):
         pK_cterm = kwargs.get('pK_cterm', {})
     elif isinstance(sequence, dict) and (('pK_nterm' in kwargs) or ('pK_cterm' in kwargs)):
         raise PyteomicsError('Can not use terminal features for %s' % type(sequence))
-    
+
     peptide_dict, pK = _prepare_charge_dict(sequence, pK=pK, pK_cterm=pK_cterm, pK_nterm=pK_nterm)
     # The algorithm is based on the fact that charge(pH) is a monotonic function.
     left_x, right_x = pI_range
@@ -416,6 +436,59 @@ different human cell types defined in a pH scale where isoelectric points
 correlate with polypeptide compositions. Electrophoresis 1994, 15, 529-539.
 """
 
+hydropathicity_KD = {
+    "A": 1.800,
+    "R": -4.500,
+    "N": -3.500,
+    "D": -3.500,
+    "C": 2.500,
+    "Q": -3.500,
+    "E": -3.500,
+    "G": -0.400,
+    "H": -3.200,
+    "I": 4.500,
+    "L": 3.800,
+    "K": -3.900,
+    "M": 1.900,
+    "F": 2.800,
+    "P": -1.600,
+    "S": -0.800,
+    "T": -0.700,
+    "W": -0.900,
+    "Y": -1.300,
+    "V": 4.200,
+}
+"""
+A set of hydropathicity indexes obtained from Kyte J., Doolittle F. J. Mol. Biol. 157:105-132 (1982).
+"""
+
+
+def gravy(sequence, hydropathicity=hydropathicity_KD):
+    """
+    Calculate GRand AVerage of hYdropathicity (GRAVY) index for amino acid sequence.
+
+    Parameters
+    ----------
+    sequence : str
+        Polypeptide sequence in one-letter format.
+    hydropathicity : dict, optional
+        Hydropathicity indexes of amino acids. Default is :py:data:`hydropathicity_KD`.
+
+    Returns
+    -------
+    out : float
+        GRand AVerage of hYdropathicity (GRAVY) index.
+
+    Examples
+    >>> gravy('PEPTIDE')
+    -1.4375
+    """
+    try:
+        return sum(hydropathicity[aa] for aa in sequence) / len(sequence)
+    except KeyError as e:
+        raise PyteomicsError("Hydropathicity for amino acid {} not provided.".format(e.args[0]))
+
+
 if __name__ == "__main__":
     import doctest
 
