@@ -77,7 +77,7 @@ Data
   :py:data:`std_labels` - a list of all standard sequence
   elements, amino acid residues and terminal modifications.
 
-  :py:data:`expasy_rules` - a dict with the regular expressions of
+  :py:data:`expasy_rules` and :py:data:`psims_rules` - two dicts with the regular expressions of
   cleavage rules for the most popular proteolytic enzymes.
 
 -------------------------------------------------------------------------------
@@ -101,7 +101,7 @@ Data
 import re
 from collections import deque
 import itertools as it
-from .auxiliary import PyteomicsError, memoize, BasicComposition
+from .auxiliary import PyteomicsError, memoize, BasicComposition, cvstr, cvquery
 
 std_amino_acids = ['Q', 'W', 'E', 'R', 'T', 'Y', 'I', 'P', 'A', 'S',
                    'D', 'F', 'G', 'H', 'K', 'L', 'C', 'V', 'N', 'M']
@@ -119,6 +119,7 @@ std_labels = std_amino_acids + [std_nterm, std_cterm]
 _nterm_mod = r'[^-]+-$'
 _cterm_mod = r'-[^-]+$'
 
+
 def is_term_mod(label):
     """Check if `label` corresponds to a terminal modification.
 
@@ -145,6 +146,7 @@ def is_term_mod(label):
     """
     return (re.match(_nterm_mod, label) or re.match(_cterm_mod, label)) is not None
 
+
 def match_modX(label):
     """Check if `label` is a valid 'modX' label.
 
@@ -158,6 +160,7 @@ def match_modX(label):
     """
     return re.match(_modX_single, label)
 
+
 def is_modX(label):
     """Check if `label` is a valid 'modX' label.
 
@@ -182,6 +185,7 @@ def is_modX(label):
     """
     return bool(match_modX(label))
 
+
 def length(sequence, **kwargs):
     """Calculate the number of amino acid residues in a polypeptide
     written in modX notation.
@@ -205,7 +209,8 @@ def length(sequence, **kwargs):
     >>> length('H-PEPTIDE-OH')
     7
     """
-    if not sequence: return 0
+    if not sequence:
+        return 0
 
     if isinstance(sequence, str) or isinstance(sequence, list):
         if isinstance(sequence, str):
@@ -219,11 +224,11 @@ def length(sequence, **kwargs):
             num_term_groups += 1
         return len(parsed_sequence) - num_term_groups
     elif isinstance(sequence, dict):
-        return sum(amount for aa, amount in sequence.items()
-                    if not is_term_mod(aa))
+        return sum(amount for aa, amount in sequence.items() if not is_term_mod(aa))
 
     raise PyteomicsError('Unsupported type of sequence.')
 
+
 def _split_label(label):
     try:
         mod, X = match_modX(label).groups()
@@ -234,11 +239,13 @@ def _split_label(label):
     else:
         return mod, X
 
+
 _modX_sequence = re.compile(r'^([^-]+-)?((?:[^A-Z-]*[A-Z])+)(-[^-]+)?$')
 _modX_group = re.compile(r'[^A-Z-]*[A-Z]')
 _modX_split = re.compile(r'([^A-Z-]*)([A-Z])')
 _modX_single = re.compile(r'^([^A-Z-]*)([A-Z])$')
 
+
 def parse(sequence, show_unmodified_termini=False, split=False, allow_unknown_modifications=False, **kwargs):
     """Parse a sequence string written in modX notation into a list of
     labels or (if `split` argument is :py:const:`True`) into a list of
@@ -333,18 +340,16 @@ def parse(sequence, show_unmodified_termini=False, split=False, allow_unknown_mo
         labels = set(labels)
         for term, std_term in zip([n, c], [std_nterm, std_cterm]):
             if term and term not in labels and not allow_unknown_modifications:
-                raise PyteomicsError(
-                            'Unknown label: {}'.format(term))
+                raise PyteomicsError('Unknown label: {}'.format(term))
         for group in parsed_sequence:
             if split:
                 mod, X = group if len(group) == 2 else ('', group[0])
             else:
                 mod, X = re.match(_modX_split, group).groups()
-            if ((not mod) and X not in labels) or not ((mod+X in labels) or (
+            if ((not mod) and X not in labels) or not ((mod + X in labels) or (
                 X in labels and (
                     mod in labels or allow_unknown_modifications))):
-                raise PyteomicsError(
-                        'Unknown label: {}'.format(group))
+                raise PyteomicsError('Unknown label: {}'.format(group))
 
     # Append terminal labels
     if show_unmodified_termini or nterm != std_nterm:
@@ -358,9 +363,9 @@ def parse(sequence, show_unmodified_termini=False, split=False, allow_unknown_mo
         else:
             parsed_sequence.append(cterm)
 
-
     return parsed_sequence
 
+
 def valid(*args, **kwargs):
     """Try to parse sequence and catch the exceptions.
     All parameters are passed to :py:func:`parse`.
@@ -377,6 +382,7 @@ def valid(*args, **kwargs):
         return False
     return True
 
+
 def fast_valid(sequence, labels=set(std_labels)):
     """Iterate over `sequence` and check if all items are in `labels`.
     With strings, this only works as expected on sequences without
@@ -396,6 +402,7 @@ def fast_valid(sequence, labels=set(std_labels)):
     """
     return set(sequence).issubset(labels)
 
+
 def tostring(parsed_sequence, show_unmodified_termini=True):
     """Create a string from a parsed sequence.
 
@@ -446,6 +453,7 @@ def tostring(parsed_sequence, show_unmodified_termini=True):
                 labels.append(''.join(cterm[:-1]))
     return ''.join(labels)
 
+
 def amino_acid_composition(sequence, show_unmodified_termini=False, term_aa=False, allow_unknown_modifications=False, **kwargs):
     """Calculate amino acid composition of a polypeptide.
 
@@ -520,6 +528,7 @@ def amino_acid_composition(sequence, show_unmodified_termini=False, term_aa=Fals
 
     return aa_dict
 
+
 @memoize()
 def cleave(sequence, rule, missed_cleavages=0, min_length=None, semi=False, exception=None):
     """Cleaves a polypeptide sequence using a given rule.
@@ -535,7 +544,7 @@ def cleave(sequence, rule, missed_cleavages=0, min_length=None, semi=False, exce
             will not work as expected.
 
     rule : str or compiled regex
-        A key present in :py:const:`expasy_rules` or a
+        A key present in :py:data:`expasy_rules`, :py:data:`psims_rules` (or an MS ontology accession) or a
         `regular expression <https://docs.python.org/library/re.html#regular-expression-syntax>`_
         describing the site of cleavage. It is recommended
         to design the regex so that it matches only the residue whose C-terminal
@@ -574,21 +583,29 @@ def cleave(sequence, rule, missed_cleavages=0, min_length=None, semi=False, exce
     True
     >>> cleave('AKAKBK', 'trypsin', 0) == {'AK', 'BK'}
     True
-    >>> cleave('GKGKYKCK', expasy_rules['trypsin'], 2) == \
+    >>> cleave('AKAKBK', 'MS:1001251', 0) == {'AK', 'BK'}
+    True
+    >>> cleave('GKGKYKCK', 'Trypsin/P', 2) == \
     {'CK', 'GKYK', 'YKCK', 'GKGK', 'GKYKCK', 'GK', 'GKGKYK', 'YK'}
     True
 
     """
     return set(_cleave(sequence, rule, missed_cleavages, min_length, semi, exception))
 
+
 def _cleave(sequence, rule, missed_cleavages=0, min_length=None, semi=False, exception=None):
     """Like :py:func:`cleave`, but the result is a list. Refer to
     :py:func:`cleave` for explanation of parameters.
     """
-    rule = expasy_rules.get(rule, rule)
+    if rule in expasy_rules:
+        rule = expasy_rules[rule]
+    elif rule in psims_rules:
+        rule = psims_rules[rule]
+    elif rule in _psims_index:
+        rule = _psims_index[rule]
     exception = expasy_rules.get(exception, exception)
     peptides = []
-    ml = missed_cleavages+2
+    ml = missed_cleavages + 2
     trange = range(ml)
     cleavage_sites = deque([0], maxlen=ml)
     if min_length is None:
@@ -603,17 +620,18 @@ def _cleave(sequence, rule, missed_cleavages=0, min_length=None, semi=False, exc
         cleavage_sites.append(i)
         if cl < ml:
             cl += 1
-        for j in trange[:cl-1]:
+        for j in trange[:cl - 1]:
             seq = sequence[cleavage_sites[j]:cleavage_sites[-1]]
             if seq and len(seq) >= min_length:
                 peptides.append(seq)
                 if semi:
-                    for k in range(min_length, len(seq)-1):
+                    for k in range(min_length, len(seq) - 1):
                         peptides.append(seq[:k])
-                    for k in range(1, len(seq)-min_length+1):
+                    for k in range(1, len(seq) - min_length + 1):
                         peptides.append(seq[k:])
     return peptides
 
+
 def num_sites(sequence, rule, **kwargs):
     """Count the number of sites where `sequence` can be cleaved using
     the given `rule` (e.g. number of miscleavages for a peptide).
@@ -642,6 +660,7 @@ def num_sites(sequence, rule, **kwargs):
     """
     return len(_cleave(sequence, rule, **kwargs)) - 1
 
+
 expasy_rules = {
     'arg-c':         r'R',
     'asp-n':         r'\w(?=D)',
@@ -682,7 +701,7 @@ expasy_rules = {
                      r'((?<=[AFGILTVM][AFGILTVWA]P)R(?=[^DE][^DE]))',
     'trypsin':       r'([KR](?=[^P]))|((?<=W)K(?=P))|((?<=M)R(?=P))',
     'trypsin_exception': r'((?<=[CD])K(?=D))|((?<=C)K(?=[HY]))|((?<=C)R(?=K))|((?<=R)R(?=[HR]))',
-    }
+}
 """
 This dict contains regular expressions for cleavage rules of the most
 popular proteolytic enzymes. The rules were taken from the
@@ -701,6 +720,44 @@ exception=parser.expasy_rules['trypsin_exception'])
         {'PEPTIDKDE'}
 """
 
+
+psims_rules = {
+    cvstr('2-iodobenzoate', 'MS:1001918'): r'(?<=W)',
+    cvstr('Arg-C', 'MS:1001303'): r'(?<=R)(?!P)',
+    cvstr('Asp-N', 'MS:1001304'): r'(?=[BD])',
+    cvstr('Asp-N ambic', 'MS:1001305'): r'(?=[DE])',
+    cvstr('CNBr', 'MS:1001307'): r'(?<=M)',
+    cvstr('Chymotrypsin', 'MS:1001306'): r'(?<=[FYWL])(?!P)',
+    cvstr('Formic acid', 'MS:1001308'): r'((?<=D))|((?=D))',
+    cvstr('Lys-C', 'MS:1001309'): r'(?<=K)(?!P)',
+    cvstr('Lys-C/P', 'MS:1001310'): r'(?<=K)',
+    cvstr('PepsinA', 'MS:1001311'): r'(?<=[FL])',
+    cvstr('TrypChymo', 'MS:1001312'): r'(?<=[FYWLKR])(?!P)',
+    cvstr('Trypsin', 'MS:1001251'): r'(?<=[KR])(?!P)',
+    cvstr('Trypsin/P', 'MS:1001313'): r'(?<=[KR])',
+    cvstr('V8-DE', 'MS:1001314'): r'(?<=[BDEZ])(?!P)',
+    cvstr('V8-E', 'MS:1001315'): r'(?<=[EZ])(?!P)',
+    cvstr('glutamyl endopeptidase', 'MS:1001917'): r'(?<=[^E]E)',
+    cvstr('leukocyte elastase', 'MS:1001915'): r'(?<=[ALIV])(?!P)',
+    cvstr('proline endopeptidase', 'MS:1001916'): r'(?<=[HKR]P)(?!P)',
+}
+"""
+This dict contains regular expressions for cleavage rules of the most
+popular proteolytic enzymes. The rules were taken from the PSI `MS ontology
+<http://purl.obolibrary.org/obo/MS_1001045>`_.
+
+You can use names or accessions to access the rules.
+Use :py:func:`pyteomics.auxiliary.cvquery` for accession access::
+
+    >>> from pyteomics.auxiliary import cvquery
+    >>> from pyteomics.parser import psims_rules
+    >>> cvquery(psims_rules, 'MS:1001918')
+    '(?<=W)'
+
+"""
+
+_psims_index = cvquery(psims_rules)
+
 def isoforms(sequence, **kwargs):
     """
     Apply variable and fixed modifications to the polypeptide and yield
@@ -768,7 +825,7 @@ def isoforms(sequence, **kwargs):
         All possible unique polypeptide sequences resulting from
         the specified modifications are yielded obe by one.
     """
-    def main(group): # index of the residue (capital letter) in `group`
+    def main(group):  # index of the residue (capital letter) in `group`
         if group[-1][0] == '-':
             i = -2
         else:
@@ -776,28 +833,38 @@ def isoforms(sequence, **kwargs):
         return len(group) + i, group[i]
 
     def apply_mod(label, mod):
-    # `label` is assumed to be a tuple (see split option of parse)
-    # unmodified termini are assumed shown
-    # if the modification is not applicable, `label` is returned
+        # `label` is assumed to be a tuple (see split option of `parse`)
+        # unmodified termini are assumed shown
+        # if the modification is not applicable, `None` is returned
         group = list(label)
         m = main(group)[0]
+        c = True  # whether the change is applied in the end
         if m == 0 and not is_term_mod(mod):
             group.insert(0, mod)
-        elif mod[0] == '-' and (group[-1] == std_cterm or (
-            group[-1][0] == '-' and override)):
+        elif mod[0] == '-' and (group[-1] == std_cterm or (group[-1][0] == '-' and override)):
             group[-1] = mod
-        elif mod[-1] == '-' and (group[0] == std_nterm or (
-            group[0][-1] == '-' and override)):
+        elif mod[-1] == '-' and (group[0] == std_nterm or (group[0][-1] == '-' and override)):
             group[0] = mod
         elif not is_term_mod(mod):
-            if m and not group[m-1][-1] == '-':
+            if m and group[m - 1][-1] != '-':
                 if override:
-                    group[m-1] = mod
+                    group[m - 1] = mod
+                else:
+                    c = False
             else:
                 group.insert(m, mod)
-        return tuple(group)
+        else:
+            c = False
+        if c:
+            return tuple(group)
 
     variable_mods = kwargs.get('variable_mods', {})
+    varmods_term, varmods_non_term = [], []
+    for m, r in sorted(variable_mods.items()):
+        if is_term_mod(m):
+            varmods_term.append((m, r))
+        else:
+            varmods_non_term.append((m, r))
     fixed_mods = kwargs.get('fixed_mods', {})
     parse_kw = {}
     if 'labels' in kwargs:
@@ -809,59 +876,71 @@ def isoforms(sequence, **kwargs):
     format_ = kwargs.get('format', 'str')
 
     # Apply fixed modifications
-    for cmod in fixed_mods:
+    for cmod, res in fixed_mods.items():
         for i, group in enumerate(parsed):
-            if fixed_mods[cmod] == True or main(group)[1] in fixed_mods[cmod]:
-                parsed[i] = apply_mod(group, cmod)
+            if res is True or main(group)[1] in res:
+                parsed[i] = apply_mod(group, cmod) or parsed[i]
 
     # Create a list of possible states for each group
     # Start with N-terminal mods and regular mods on the N-terminal residue
-    second = set(apply_mod(parsed[0], m) for m, r in variable_mods.items()
-                if (r == True or
-                    main(parsed[0])[1] in r or
-                    'nterm' + main(parsed[0])[1] in r or
-                    (len(parsed) == 1 and 'cterm' + main(parsed[0])[1] in r))
-                and not is_term_mod(m)
-                ).union([parsed[0]])
-    first = it.chain((apply_mod(group, mod) for group in second
-            for mod, res in variable_mods.items()
-        if (mod.endswith('-') or (mod.startswith('-') and len(parsed) == 1))
-        and (res == True or main(group)[1] in res)), second)
-    states = [[parsed[0]] + list(set(first).difference({parsed[0]}))]
+    states = [[parsed[0]]]
+    m0 = main(parsed[0])[1]
+    for m, r in varmods_non_term:
+        if r is True or m0 in r or 'nterm' + m0 in r or len(parsed) == 1 and 'cterm' + m0 in r:
+            applied = apply_mod(parsed[0], m)
+            if applied is not None:
+                states[0].append(applied)
+    more_states = []
+    for m, r in varmods_term:
+        if r is True or m0 in r:
+            if m[-1] == '-' or len(parsed) == 1:
+                for group in states[0]:
+                    applied = apply_mod(group, m)
+                    if applied is not None:
+                        more_states.append(applied)
+    states[0].extend(more_states)
+
     # Continue with regular mods
-    states.extend([group] + list(set(apply_mod(group, mod)
-            for mod in variable_mods if (
-                variable_mods[mod] == True or
-                group[-1] in variable_mods[mod]) and not is_term_mod(mod)
-                ).difference({group}))
-        for group in parsed[1:-1])
+    for group in parsed[1:-1]:
+        gstates = [group]
+        for m, r in varmods_non_term:
+            if r is True or group[-1] in r:
+                applied = apply_mod(group, m)
+                if applied is not None:
+                    gstates.append(applied)
+        states.append(gstates)
+
     # Finally add C-terminal mods and regular mods on the C-terminal residue
     if len(parsed) > 1:
-        second = set(apply_mod(parsed[-1], m) for m, r in variable_mods.items()
-                    if (r == True or
-                        main(parsed[-1])[1] in r or
-                        'cterm' + main(parsed[-1])[1] in r)
-                    and not is_term_mod(m)
-                ).union((parsed[-1],))
-        first = it.chain((apply_mod(group, mod) for group in second
-                for mod, res in variable_mods.items()
-            if mod.startswith('-') and (
-                res == True or main(group)[1] in res)), second)
-        states.append([parsed[-1]] + list(set(first).difference({parsed[-1]})))
+        states.append([parsed[-1]])
+        m1 = main(parsed[-1])[1]
+        for m, r in varmods_non_term:
+            if r is True or m1 in r or 'cterm' + m1 in r or len(parsed) == 1 and 'nterm' + m1 in r:
+                applied = apply_mod(parsed[-1], m)
+                if applied is not None:
+                    states[-1].append(applied)
+        more_states = []
+        for m, r in varmods_term:
+            if r is True or m1 in r:
+                if m[0] == '-' or len(parsed) == 1:
+                    for group in states[-1]:
+                        applied = apply_mod(group, m)
+                        if applied is not None:
+                            more_states.append(applied)
+        states[-1].extend(more_states)
 
     sites = [s for s in enumerate(states) if len(s[1]) > 1]
     if max_mods is None or max_mods > len(sites):
         possible_states = it.product(*states)
     else:
         def state_lists():
-            for m in range(max_mods+1):
+            for m in range(max_mods + 1):
                 for comb in it.combinations(sites, m):
                     skel = [[s[0]] for s in states]
                     for i, e in comb:
                         skel[i] = e[1:]
                     yield skel
-        possible_states = it.chain.from_iterable(
-                it.product(*skel) for skel in state_lists())
+        possible_states = it.chain.from_iterable(it.product(*skel) for skel in state_lists())
 
     if format_ == 'split':
         def strip_std_terms():
@@ -880,6 +959,7 @@ def isoforms(sequence, **kwargs):
     else:
         raise PyteomicsError('Unsupported value of "format": {}'.format(format_))
 
+
 def coverage(protein, peptides):
     """Calculate how much of `protein` is covered by `peptides`.
     Peptides can overlap. If a peptide is found multiple times in `protein`,
@@ -914,7 +994,7 @@ def coverage(protein, peptides):
         indices = [m.start() for m in re.finditer(
             '(?={})'.format(re.sub(r'[^A-Z]', '', peptide)), protein)]
         for i in indices:
-            mask[i:i+len(peptide)] = 1
+            mask[i:i + len(peptide)] = 1
     return mask.sum(dtype=float) / mask.size
 
 
